# -*- coding: utf-8 -*-
"""
Module dllt.py
===========================================

Module to represent the **Doubly Linked List** data structure in *PyDASA*.

This module provides the **DoubleLinkedList** class which implements a doubly linked list with methods for insertion, deletion, and traversal.

classes:
    **DoubleLinkedList**: Implements a doubly linked list with methods for insertion, deletion, and traversal.

*IMPORTANT:* based on the implementations proposed by the following authors/books:

    #. Algorithms, 4th Edition, Robert Sedgewick and Kevin Wayne.
    #. Data Structure and Algorithms in Python, M.T. Goodrich, R. Tamassia, M.H. Goldwasser.
"""
# TODO complete the DoubleLinkedList implementation
