# -*- coding: utf-8 -*-
"""
Module units.py
===========================================

This module provides the Measurement class for handling physical measurements with units, values, and validation.

Classes:
    **Measurement**: Represents a physical measurement with a value, unit, and validation logic.

*IMPORTANT:* Based on the theory from:

    # H.Gorter, *Dimensionalanalyse: Eine Theoririe der physikalischen Dimensionen mit Anwendungen*
"""
# TODO complete implementation of Measurement class
